#!/usr/bin/tclsh
package require websocket
if {[catch {package require twapi_crypto}]} {
    package require tls 1.7
    http::register https 443 [list ::tls::socket -autoservername true]
} else {
    http::register https 443 [list ::twapi::tls_socket]
}
package require json

oo::class create Huobi {
variable URL
variable sock 
constructor { { url wss://api-aws.huobi.pro/ws } } {
	set URL $url
	set sock [ my Connect ]
}
method Connect {} {
	set sock [ websocket::open $URL [ list [ self ] Handler ] -timeout 10000 ]
	puts "Socket $sock"
}
method Handler { s type msg } {
    switch -glob -nocase -- $type {
		co* {
			set sock $s
			my OnConnect
		}
		te* {
			my OnMessage $msg
		}
		bin* {
			my OnBinary $msg
		}
		cl* -
		dis* {
			my OnDisconnect $msg
		}
		tim* {
			puts "Timeout!!!"
		} default {
			puts "Message $type $msg"
		}
	}
}
method OnConnect {} {
	puts "Connected :-)"
	set subscribe {{"sub":"market.btcusdt.bbo","id":"1132"}}
	puts "subscribe:: $subscribe"
	websocket::send $sock text $subscribe
}
method OnDisconnect { msg } {
	puts "Disconnect :-("
}
method OnBinary { bin } {
	set msg [ zlib gunzip $bin ]
	my OnMessage $msg
}
method OnMessage { msg } {
	set data [ json::json2dict $msg ]
	if { [ dict exists $data "ping" ] } {
		set json [ format {{"pong":%s}} [ dict get $data "ping" ] ]
		puts "pong:: $json"
		websocket::send $sock text $json
	} else {
		puts $msg
	}
}
export Handler OnConnect OnDisconnect OnMessage
}; #!/class WSClient

set huobi [ Huobi new ]

vwait the_end
